package nstat.patch;

import   sep.operator.*;
import   jam.operator.*; 
import   jam.vector.*;
import   rsf.vector.*;
import   rsf.operator.*; 

/**
 * This operator breaks an output quilt into patches, 
 * computes corresponding input patches, images these input patches with 
 * stationary operators, and merges the image patches into the image quilt. 
 * 
 * @see PatcherFactory
 * @see PatchPushPullTest
 * @see PatchPush
 * 
 * <BR><!--#include file="./source.inc"--><BR>
 */ 
public class PatchPullOp extends SepOperator {
  float[]   overlap; 
  int  []   patchsize;
  OperatorFactory opFac;
  /** quilt weight */
  Rsf wght; 
  /** patch weight */ 
  Rsf wptc; 

  /** 
   * @param range     output quilt space 
   * @param overlap   desired overlap
   * @param patchsize 
   * @param opFac     constructs the stationary operator for given patch space.
   */
  public PatchPullOp(RsfSpace range, 
		     float   overlap, int[] patchsize,
		     OperatorFactory opFac){
    this((RsfSpace) opFac.createDomain(range), range,
      juice.util.JuiceUtil.arrayfill(range.getDimension(),overlap),patchsize,
      opFac);
  }
  public PatchPullOp(RsfSpace domain, RsfSpace range, 
		     float[] overlap,int[] patchsize,
		     OperatorFactory opFac){ 
    super(domain, range); 
    this.overlap   = overlap; 
    this.patchsize = patchsize; 
    this.opFac     = opFac; 

    this.wght = (Rsf) range.newMember(); 
    this.wptc = (Rsf) RsfFactory.newRsf(patchsize);
    rsf.util.Tent.setTent(this.wptc); 
  }

  public void apply(boolean add, Vector iput, Vector oput) {
    if (add) System.err.println("Not implemented!"); 
    // should test if iput is memeber of domain and oput member of range // 
    PatcherFactory f   = 
      new PatcherFactory((RsfSpace) oput.getSpace(), patchsize, overlap);
    Patcher         pchr = f.getPatcher();
    for (int i=1; pchr.hasMoreElements(); i++) {
      //System.err.println("Doing patch #" + i + " of " + pchr.getNPatches());
      Window        zpad = (Window) ((Window) pchr.nextElement()).adjoint();
      Operator      oper = opFac.getOpWithRange(zpad.getDomain()); 
      WindowFactory wfac = new WindowFactory((RsfSpace) iput.getSpace(),
					     (RsfSpace) oper.getDomain());
      Window        trun = wfac.getForward();
      Rsf           iptc  = (Rsf) trun.image(iput); 
      Rsf           optc  = (Rsf) oper.image(iptc);
      optc.mul(wptc);
      zpad.addImage(optc,oput);
      zpad.addImage(wptc,wght); //Window Operator ignores Space consistency!//
    }
    // I should not write out in a standard application // 
    ((Rsf) oput).write("pullStck.H");
    ((Rsf) wght).write("pullWght.H");
    rsf.operator.ElwDiv.elwDiv((Rsf) oput, (Rsf) oput,wght);
  }
  public jam.JamObject copy() {
    System.err.println("I am sick of deep copies of operators! Matt!");
    return new PatchPullOp((RsfSpace) domain, (RsfSpace) range, 
			   overlap, patchsize, opFac); 
  }
}



