package nstat.patch;

import   sep.operator.*;
import   jam.operator.*; 
import   jam.vector.*;
import   rsf.vector.*;
import   rsf.operator.*; 

/**
 * This operator breaks an input quilt into patches, images the patches with 
 * stationary operators, and merges the image patches into a image quilt. 
 * 
 * @see PatcherFactory
 * @see PatchPushPullTest
 * @see PatchPull 
 * 
 * <BR><!--#include file="./source.inc"--><BR>
 */ 
public class PatchPushOp extends SepOperator {
  float[]   overlap; 
  int  []   patchsize;
  OperatorFactory opFac;
  /** quilt weight */
  Rsf wght; 
  /** patch weight */ 
  Rsf wptc; 

  public PatchPushOp(RsfSpace domain, float   overlap, int[] patchsize,
		 OperatorFactory opFac){
    this(domain,juice.util.JuiceUtil.arrayfill(domain.getDimension(),overlap),
	 patchsize, opFac);
  }
  /** 
   * @param domain    input quilt space 
   * @param overlap   desired overlap
   * @param patchsize 
   * @param opFac     constructs the stationary operator for given patch space.
   */
  public PatchPushOp(RsfSpace domain, float[] overlap,int[] patchsize,
		 OperatorFactory opFac){ 
    super(domain, opFac.createRange(domain)); 
    this.overlap   = overlap; 
    this.patchsize = patchsize; 
    this.opFac     = opFac; 

    this.wght = (Rsf) range.newMember(); 
    this.wptc = (Rsf) opFac.createRange(new RsfSpace(patchsize)).newMember();
    rsf.util.Tent.setTent(this.wptc); 
  }

  public void apply(boolean add, Vector iput, Vector oput) {
    // should test if iput is memeber of domain // 
    PatcherFactory f   = 
      new PatcherFactory((RsfSpace) iput.getSpace(), patchsize, overlap);
    Patcher         pchr = f.getPatcher();
    for (int i=1; pchr.hasMoreElements(); i++) {
      //System.err.println("Doing patch #" + i + " of " + pchr.getNPatches());
      Rsf           iptc = pchr.nextPatch((Rsf) iput); 
      Operator      oper = opFac.getOpWithDomain(iptc.getSpace()); 
      Rsf           optc = (Rsf) oper.image(iptc);
      WindowFactory wfac = new WindowFactory((RsfSpace) oput.getSpace(),
					   (RsfSpace) optc.getSpace());
      Window        zpad = wfac.getAdjoint();
      optc.mul(wptc);
      zpad.addImage(optc,oput);
      zpad.addImage(wptc,wght); //Window Operator ignores Space consistency!//
    }
    // I should not write out in a standard application // 
    ((Rsf) oput).write("pushStck.H");
    ((Rsf) wght).write("pushWght.H");
    rsf.operator.ElwDiv.elwDiv((Rsf) oput, (Rsf) oput,wght);
  }
  public jam.JamObject copy() {
    System.err.println("I am sick of deep copies of operators! Matt!");
    return new PatchPushOp((RsfSpace) domain, overlap, patchsize, opFac); 
  }
}



