package nstat.patch;

import   juice.util.Tester;
import   juice.vector.*;
import   jam.vector.*;
import   jam.operator.*; 
import   rsf.vector.*;

public class PatchPushPullTest { 
  public static void main(String[] args) {
    // ----- create the input ----- // 
    Rsf iput = PatchPushPullTest.createInput();
    float overlap   =  0.3f;
    int[] patchsize = {101};
    boolean correct =  true;
    // this is an operator I created for testing purpose (see below) // 
    if(! StatVarHalfTest.test()) correct = false; 
    // ----- processing PatchPushOp----- // 
    iput.write("pushIput.H");
    OperatorFactory opFac1 = new StatVarHalfFactory(); 
    PatchPushOp     pchOp1 = new PatchPushOp((RsfSpace) iput.getSpace(), 
					     overlap, patchsize, opFac1);
    Rsf oput1 = (Rsf) pchOp1.image(iput);
    oput1.write("pushOput.H");
    oput1.write("junk1.H"); 
    if (!testPatchPushOp(oput1)) correct = false;
    // ----- processing PatchPullOp----- // 
    // this is an operator I created for testing purpose (see below) // 
    iput.write("pullIput.H");
    OperatorFactory opFac2 = new StatVarHalfFactory(); 
    PatchPullOp     pchOp2 = new PatchPullOp(
				(RsfSpace) opFac2.createRange(iput.getSpace()),
				overlap, patchsize, opFac2);
    Rsf oput2 = (Rsf) pchOp2.image(iput);
    oput2.write("pullOput.H");
    oput2.write("junk2.H"); 
    if (!testPatchPullOp(oput2)) correct = false;

    Tester.exit(correct); 
  } 
  /** 
   * Rsf1 filled with random numbers. Variance varies but is locally constant.
   */
  static Rsf createInput() {
    Rsf1 iput        = (Rsf1) RsfFactory.newRsf(1201);
    java.util.Random randomizer = new java.util.Random(1925l); 
    iput.rand(randomizer);
    for (int i=0  ; i<300 ; i++) 
      iput.setDatum(i,iput.getDatum(i)*0.1f); 
    for (int i=301; i<600 ; i++) 
      iput.setDatum(i,iput.getDatum(i)*1.0f); 
    for (int i=601; i<900 ; i++) 
      iput.setDatum(i,iput.getDatum(i)*0.3f); 
    for (int i=900; i<1200; i++) 
      iput.setDatum(i,iput.getDatum(i)*0.1f); 
    return iput; 
  }
  static boolean testPatchPushOp(Rsf oput) {
    System.err.println("PatchPush:");
    boolean correct = true; 
    float[] d = ((Rsf1) oput).d; 
    float   f = d[75]; 
    System.err.println("0.001 < " + f + " < 0.01"); 
    if (f < 0.001f || f > 0.01f) correct = false; 
    f = d[225];
    System.err.println("0.1   < " + f + " < 1   "); 
    if (f < 0.1f   || f > 1f   ) correct = false; 
    f = d[375];
    System.err.println("0.01  < " + f + " < 0.1 "); 
    if (f < 0.01f || f > 0.1f  ) correct = false; 
    f = d[525];
    System.err.println("0.001 < " + f + " < 0.01"); 
    if (f < 0.001f || f > 0.01f) correct = false; 
    System.err.println(correct? "passed":"failed"); 
    return correct; 
  }
  static boolean testPatchPullOp(Rsf oput) {
    System.err.println("PatchPull:");
    boolean correct = true; 
    float[] d = ((Rsf1) oput).d; 
    float   f = d[50]; 
    System.err.println("0.001 < " + f + " < 0.01"); 
    if (f < 0.001f || f > 0.01f) correct = false; 
    f = d[225];
    System.err.println("0.1   < " + f + " < 1   "); 
    if (f < 0.1f   || f > 1f   ) correct = false; 
    f = d[375];
    System.err.println("0.01  < " + f + " < 0.1 "); 
    if (f < 0.01f || f > 0.1f  ) correct = false; 
    f = d[525];
    System.err.println("0.001 < " + f + " < 0.01"); 
    if (f < 0.001f || f > 0.01f) correct = false; 
    System.err.println(correct? "passed":"failed"); 
    return correct; 
  }
}

/* This was before an independent file but I use it now only for testing */
//package  rsf.operator;
//
//import   jam.vector.*;
//import juice.vector.*;
//import   rsf.vector.*;

/**
 * <BR><!--#include file="./source.inc"--><BR>
 *
 * This is just a stupid test class. 
 * 
 * Computes the variance of an input Rsf: Sum(x_i^2 - \bar{a}^2) \br 
 * All the elements of the output Rsf are constant value. 
 * 
 * The output space is derived from the input space and vice versa.
 */
class StatVarHalf implements jam.operator.Operator {
  RsfSpace domain;
  RsfSpace range;

  StatVarHalf(Space domain, Space range) {
    this.domain = (RsfSpace) domain;
    this.range  = (RsfSpace) range; 
  }
  StatVarHalf(Space domain) {
    this(domain, RealSpc.getSpace()); 
  }
  StatVarHalf(Space spc, String spcName) {
    if (spcName.equals("domain")) {
      this.domain = (RsfSpace) spc; 
      this.range  = createRange((RsfSpace) spc); 
    }
    else {
      this.domain = createDomain((RsfSpace) spc); 
      this.range  = (RsfSpace) spc;
    } 
  }

  static RsfSpace createRange(RsfSpace domain) {
    int n = domain.getDimension();
    Axis[] rangeAxes = new Axis[n];
    for (int i=0; i<n; i++) {
      rangeAxes[i] = new Axis(domain.getAxes()[i]);
      rangeAxes[i].nsamples = rangeAxes[i].nsamples / 2    ;
      rangeAxes[i].delta    = rangeAxes[i].delta    * 2.0f ;
      rangeAxes[i].offset   = rangeAxes[i].offset   - 0.2f ;      
    }
    return new RsfSpace(rangeAxes);
  }

  static RsfSpace createDomain(RsfSpace range) {
    int n = range.getDimension();
    Axis[] domainAxes = new Axis[n];
    for (int i=0; i<n; i++) {
      domainAxes[i] = new Axis(range.getAxes()[i]);
      domainAxes[i].nsamples = domainAxes[i].nsamples * 2    ;
      domainAxes[i].delta    = domainAxes[i].delta    / 2.0f ;
      domainAxes[i].offset   = domainAxes[i].offset   + 0.2f ;      
    }
    return new RsfSpace(domainAxes);
  }

  // operator superclass methods

  public jam.JamObject copy  () { return new StatVarHalf(domain, range); }

  public Vector image(Vector d) {
    Vector r = getRange().newMember();
    statVarHalf(false,r,d);
    return r;
  }
  public void image(Vector d, Vector r){
    statVarHalf(false,r,d);
  }
  /** 
   * I wonder if this method makes any sense for Var(). I 
   * throw an error instead for now. 
   */
  public void addImage(Vector d, Vector r){ 
    statVarHalf(true,r,d);
  } 
  public void residual(Vector d, Vector b, Vector r) {
    r.neg(b);
    addImage(d,r); 
  }

  public  Space getDomain         () { return domain; }
  public  Space getRange          () { return range ; }

  public static void main(String[] args) { 
    Rsf data = RsfFactory.newRsf(System.in);
    data.fill(StatVarHalf.statVarHalf(data)); 
    data.write(System.out); 
  }

  public final static void statVarHalf(boolean add, Vector r, Vector d) {
    if (add) System.err.println("add flag not implemented!"); 
    if (d.getClass().getName().equals("RealVec")) 
      ((RealVec)r).setValue(statVarHalf((Rsf) d));
    else 
      ((Rsf)    r).fill    (statVarHalf((Rsf) d));
  }

  public final static float statVarHalf(Rsf d) {
    int[] dd   = ((RsfSpace) d.getSpace()).getNSamples();
    float var  = 0.f; 
    float mean = rsf.operator.StatMean.statMean(d); 
    
    switch (dd.length) {
    case 1: 
      float[] d0 = ((Rsf1) d).d;
      for (int i0=0; i0<dd[0]; i0++) {
	var += d0[i0]         * d0[i0];
      }
      break;
    case 2: 
      float[][] d1 = ((Rsf2) d).d;
      for (int i1=0; i1<dd[1]; i1++) {
      for (int i0=0; i0<dd[0]; i0++) {
	var += d1[i1][i0]     * d1[i1][i0];
      }}
      break;
    case 3:
      float[][][] d2 = ((Rsf3) d).d;
      for (int i2=0; i2<dd[2]; i2++) {
      for (int i1=0; i1<dd[1]; i1++) {
      for (int i0=0; i0<dd[0]; i0++) {
	var += d2[i2][i1][i0] * d2[i2][i1][i0]; 
      }}}
      break;
    }
    var = var / d.size();
    var = var - mean*mean; 
    return var;
  }
}

/* This was before an independent file but I use it now only for testing */
//package  rsf.operator;

//import 	  jam.vector.*;
//import 	  jam.operator.*;
//import 	  rsf.vector.*;

/**
 * <BR><!--#include file="./source.inc"--><BR>
 */
class StatVarHalfFactory implements OperatorFactory {
  protected StatVarHalf op;

  StatVarHalfFactory(Space domain, Space range ){ 
    this.op = new StatVarHalf(domain, range);
  }
  /*
  StatVarHalfFactory(Space space, String   rangeOrDomain){
    if (rangeOrDomain.equals("domain")) {
      this.op = new StatVarHalf(space, "domain");
    }
    else { 
      this.op = new StatVarHalf(space, "range");
    }
  }
  */
  /** used in app */ 
  StatVarHalfFactory() {}
  /** used in app 
   * The createRange and createDomain methods should really not rely 
   * on the operator itself. We usually have Factories because the 
   * creation of the object is difficult and should be isolated. 
   * On the other hand, when the operator is simple and we use a Factory
   * only to create the operator in an interface defined way ... 
   */ 
  public Space createRange (Space domain){
    return StatVarHalf.createRange((RsfSpace) domain);
  }
  public Space createDomain(Space range ){
    return StatVarHalf.createDomain((RsfSpace) range);
  }
  public Operator getOp(Space domain, Space range ){
    return new StatVarHalf(domain, range);
  }
  /*
  public Operator getOp() {
    return op; 
  }
  public Operator getOp(Space space, String rangeOrDomain){
    if (rangeOrDomain.equals("domain")) {
      this.op = new StatVarHalf(space, "domain"); 
    }
    else {
      this.op = new StatVarHalf(space, "range");
    } 
    return op;
  }
  */
  public Operator getOpWithRange (Space range ) {
    return new StatVarHalf(range, "range");
  }
  public Operator getOpWithDomain(Space domain) {
    return new StatVarHalf(domain, "domain"); 
  }
}

// package  rsf.operator;
//
// import   rsf.vector.*;
// import   juice.util.Tester;

/**
 * <BR><!--#include file="./source.inc"--><BR>
 */
class StatVarHalfTest {
  static boolean test() {

    boolean correct = true;

    int[] dd = {2,3};
    Rsf data = RsfFactory.newRsf(dd);
    int[] rr = {3};
    Rsf rslt = RsfFactory.newRsf(rr);

    data.zero(); 
    data.fill(1.0f); 
    System.out.println(data);

    // uses the static routine 
    System.out.println("This should be 0: " + StatVarHalf.statVarHalf(data));
    if (StatVarHalf.statVarHalf(data) != 0.0f) correct = false;

    data.zero();
    int[] i0 = {0,0};
    data.setDatum(i0,3.0f);
    int[] i1 = {1,0};
    data.setDatum(i1,3.0f);
    System.out.println(data);

    // uses the static routine 
    System.out.println("This should be about 2f: " + StatVarHalf.statVarHalf(data));
    if (StatVarHalf.statVarHalf(data) != 2.0f) correct = false;

    data.zero();
    data.setDatum(i0,6.0f);
    System.out.println(data);

    // uses the static routine 
    System.out.println("This should be 5: " + StatVarHalf.statVarHalf(data));
    if (StatVarHalf.statVarHalf(data) != 5.0f) correct = false;

    // uses the operator entry 
    StatVarHalf myVar = new StatVarHalf(data.getSpace(),rslt.getSpace()); 
    myVar.image(data,rslt); 

    float[] y = ((Rsf1) rslt).d;
    for(int i=0; i<rr[0]; i++) {
      if (y[i] != 5.0f) correct = false;
    }

    System.out.println("The next array elements are all of value 5 ");
    System.out.println(rslt);

    java.util.Random randomizer = new java.util.Random(1901l); 
    data.rand(randomizer);
    myVar.image(data,rslt); 
    
    System.out.println("This should be a 3 element Rsf filled with 0.116748:");
    System.out.println(rslt);
    rslt.write("junkyme.H");

    float[] z = ((Rsf1) rslt).d;
    for(int i=0; i<z.length; i++) {
        if (z[i] > 0.11675f || z[i] < 0.11674f ) correct = false;
    }

    StatVarHalf myVarHalf = new StatVarHalf(data.getSpace(), "domain");
    rslt = (Rsf) myVarHalf.getRange().newMember();
    System.out.println(data); 
    System.out.println(rslt);
    myVarHalf.image(data,rslt);
    System.out.println(rslt);

    /*
    StatVarHalfFactory statFac = new StatVarHalfFactory(data.getSpace(),"domain");
    StatVarHalf varHalf1 = (StatVarHalf) statFac.getOp(); 
    */
    StatVarHalfFactory statFac = new StatVarHalfFactory();
    StatVarHalf varHalf1 = (StatVarHalf) statFac.getOpWithDomain(data.getSpace());
    
    rslt = (Rsf) varHalf1.getRange().newMember();
    varHalf1.image(data,rslt);
    System.out.println(rslt);

    System.err.println(correct? "passed":"failed"); 
    return correct; 
  }
}








