package  nstat.patch;

import  juice.util.JuiceUtil;
import  juice.util.Tester;
import    rsf.vector.*;

/**
 * <BR><!--#include file="./source.inc"--><BR>
 *
 * @author Matthias Schwab 
 */
public class PatcherTest {
  public static void main(String[] args) {

    boolean correct = true;

    int[] quiltSize = { 100, 120 };
    int[] patchSize = {  21,  25 };

    // create the data input
    RsfSpace quiltSpace = new RsfSpace(quiltSize);
    Rsf quilt = RsfFactory.newRsf(quiltSpace);
    quilt.fill(1F);
    // create the data output
    Rsf out   = RsfFactory.newRsf(quiltSpace);
    out.zero();

    float[] ow = { .1f, .3f};
    PatcherFactory pwFac = new PatcherFactory(quiltSpace, patchSize, ow);
    Patcher pw = pwFac.getPatcher(); 
    // Prints out some stats .. 
    System.out.println("QuiltSpace: \n" + quiltSpace + "\n");
    System.out.println("numPatches: "  + pw.getNPatches     (0) +
		       ", "            + pw.getNPatches     (1));
    if (pw.getNPatches(0) != 5) correct = false;   
    if (pw.getNPatches(1) != 6) correct = false;   
    System.out.println("overlap   : " + pwFac.getOverlap(0) +
		       ", "           + pwFac.getOverlap(1));
    if (JuiceUtil.approx(pwFac.getOverlap(0),0.0595238f)) correct=false; 
    if (JuiceUtil.approx(pwFac.getOverlap(1),0.24f     )) correct=false; 
    for ( ; pw.hasMoreElements(); ) {
      System.out.println("Anchor: " +
			 JuiceUtil.arrayToString(((Window) pw.nextElement())
					    .getAnchor(), ", "));
    }
 
    if (pw.getNPatches(0) != 5) correct = false;   
    if (pw.getNPatches(1) != 6) correct = false;   

    int[] index0 = {0,0};
    if (pw.getAnchor(index0)[0] != 0 )  correct = false; 
    if (pw.getAnchor(index0)[1] != 0 )  correct = false;
    int[] index1 = {4,5}; // last anchor (pw.getNPatches(i)-1) 
    if (pw.getAnchor(index1)[0] != 79 ) correct = false; 
    if (pw.getAnchor(index1)[1] != 95 ) correct = false;
 
    Patcher pw2 = pwFac.getPatcher(); 
    // cutting and pasting of the windows
    for (; pw2.hasMoreElements(); ) {
      Window cut   = (Window) pw2.nextElement();
      Window paste = (Window) cut.adjoint();
      paste.addImage(cut.image(quilt), out);
    }
    // lets have a look at the first window
    Window cut  = (Window) pw.getWindow(index0);
    Rsf2  patch = (Rsf2) cut.image(quilt); 
    System.out.println("Here is the first patch: \n" + patch); 
    if (patch.d[0][0] != 1.f)           correct = false; 
    int[] sample = ((RsfSpace) patch.getSpace()).getNSamples();
    System.out.println("The 1st axis should have 21 samples: " + sample[0]);
    if (sample[0] != 21)                correct = false; 
    System.out.println("The 2nd axis should have 25 samples: " + sample[1]);
    if (sample[1] != 25)                correct = false; 

    // lets have a look at its adjoint
    Window paste = (Window) cut.adjoint();
    Rsf2 quilt2   = (Rsf2) paste.image(cut.image(quilt)); 
    System.out.println("Here is the quilt2 Rsf: \n" + quilt2); 
    if (quilt2.d[00][00] != 1.f) correct = false;
    if (quilt2.d[24][20] != 1.f) correct = false;
    if (quilt2.d[25][21] != 0.f) correct = false;
    if (quilt2.d[00][21] != 0.f) correct = false;
    if (quilt2.d[25][00] != 0.f) correct = false;

    System.out.println(out);
    out.write("junk.H");

    Tester.exit(correct); 
  }
}

