package  nstat.patch;

import 	  jam.vector.*;
import 	  jam.operator.*;
import  juice.util.JuiceUtil;
import    rsf.vector.*;
import  juice.util.Tester;

/**
 * <BR><!--#include file="./source.inc"--><BR>
 */
final
public class WindowTest {
  public static void main( String[] args) {

    boolean passed = true;

    // test Tent values

    // create a wall space
    Axis[] ax0 = new Axis[1];
    ax0[0] = new Axis( 11, 2f, 0f, "distance", "m" );
    Rsf1 wall01 = (Rsf1)RsfFactory.newRsf(ax0);
    wall01.zero();
    int m0 = ((RsfSpace) wall01.getSpace()).getNSamples(0);
    for (int i = 0; i < m0; i++)
      wall01.d[i] = (float) i;

    int[] windSize0 = { 11 };
    int[] anchor0   = { 0  };
    Window myWind0 =
      new WindowFactory((RsfSpace) wall01.getSpace(),
		       anchor0, windSize0).getForward();

    /*
    Rsf1 tent01 = (Rsf1) myWind0.getWindowTent();
    System.out.println("tent01: " + tent01); 

    float eps = 1.0e-6f; 
    if (tent01.d[0]  > (0.0277778f + eps)) passed = false;
    if (tent01.d[0]  < (0.0277778f - eps)) passed = false;
    if (tent01.d[5]  > 0.1666667f + eps) passed = false;
    if (tent01.d[5]  < 0.1666667f - eps) passed = false;
    if (tent01.d[10] > 0.0277778f + eps) passed = false;
    if (tent01.d[10] < 0.0277778f - eps) passed = false;
    */

    // test operator 

    // create a wall space
    Axis[] ax = new Axis[1];
    ax[0] = new Axis( 4, 2f, 0f, "distance", "m" );
    Rsf1 wall11 = (Rsf1)RsfFactory.newRsf(ax);
    wall11.zero();
    int n0 = ((RsfSpace) wall11.getSpace()).getNSamples(0);
    for (int i = 0; i < n0; i++)
      wall11.d[i] = (float) i;

    // create a Window operator and compute its image
    int[] windSize1 = { 2 };
    int[] anchor1    = { 1 };
    Window myWind1 =
      new WindowFactory((RsfSpace) wall11.getSpace(),
		       anchor1, windSize1).getForward();
    Rsf1 wind11 = (Rsf1) myWind1.image(wall11);
    float o11 = ((RsfSpace) wind11.getSpace()).getAxes()[0].offset; 

    System.out.println("test Window forward: \n");
    if (wind11.d[0] !=  1F) passed = false;
    if (wind11.d[1] !=  2F) passed = false;
    System.out.println("WallRsf : " + wall11);
    System.out.println("WindRsf : " + wind11);
    System.out.println("Wind size, anchor = " +
		       windSize1[0] + " " + anchor1[0]);

    /*
    System.out.println("test tentweight: \n");
    Rsf1  tent11 = (Rsf1)      myWind1.getWindowTent();
    float o13   = ((RsfSpace) tent11.getSpace()).getAxes()[0].offset; 
    if (tent11.d[0] !=  0.5F) passed = false;
    if (tent11.d[1] !=  0.5F) passed = false;
    System.out.println(tent11);
    */

    System.out.println("test moving setting the anchor");

    // move the offset 
    int[] anchor2    = { 0 };
    myWind1.setAnchor(anchor2);
    Rsf1 wind12 = (Rsf1) myWind1.image(wall11);
    float o12 = ((RsfSpace) wind12.getSpace()).getAxes()[0].offset;

    // compare values and offset (before and after)
    if (wind12.d[0] !=  0F) passed = false;
    if (wind12.d[1] !=  1F) passed = false;
    if (o11 == o12) passed = false; 

    System.out.println("WindRsf : " + wind12);
    System.out.println("The two offsets should be different:");
    System.out.println("wind11 offset should be 2: " + o11); 
    System.out.println("wind12 offset should be 0: " + o12);

    /*
    System.out.println("tent has new offsets");
    System.out.println("But the weights were not recomputed");

    Rsf1 tent12 = (Rsf1) myWind1.getWindowTent();
    float o14 = ((RsfSpace) tent12.getSpace()).getAxes()[0].offset;

    if (o13 == o14) passed = false; 
    System.out.println("The two offsets should be different:");
    System.out.println("tent11 offset should be 2: " + o13); 
    System.out.println("tent12 offset should be 0: " + o14);
 
    System.out.println(tent12);
    */

    // 3D test

    // create a wall space
    Axis[] ax3 = new Axis[3];
    ax3[0] = new Axis( 3, 1f, -4f, "time", "s" );
    ax3[1] = new Axis( 4, 2f, 12f, "distance", "m" );
    ax3[2] = new Axis( 5, 3f,  6f, "money", "$" );
    Rsf3 wall31   = (Rsf3) RsfFactory.newRsf(ax3);
    float o20   = ((RsfSpace) wall31.getSpace()).getAxes()[0].offset; 
    float o21   = ((RsfSpace) wall31.getSpace()).getAxes()[1].offset; 
    float o22   = ((RsfSpace) wall31.getSpace()).getAxes()[2].offset; 
    wall31.zero();
    int[] nn = ((RsfSpace) wall31.getSpace()).getNSamples();
    for     (int i2 = 0; i2 < nn[2]; i2++)
      for   (int i1 = 0; i1 < nn[1]; i1++)
	for (int i0 = 0; i0 < nn[0]; i0++)
	  wall31.d[i2][i1][i0] = i0 + 10f*i1 + 100f*i2;

    // create a Window operator and compute its image
    int[] windSize3  = { 2, 2, 3 };
    int[] anchor3    = { 1, 2, 0 };
    Window myWind3 = new WindowFactory((RsfSpace) wall31.getSpace(),
			              anchor3, windSize3).getForward();
    Rsf3 wind31 = (Rsf3) myWind3.image(wall31);
    float o30   = ((RsfSpace) wind31.getSpace()).getAxes()[0].offset; 
    float o31   = ((RsfSpace) wind31.getSpace()).getAxes()[1].offset; 
    float o32   = ((RsfSpace) wind31.getSpace()).getAxes()[2].offset; 

    System.out.println("WallRsf  : " + wall31);
    System.out.println("WindRsf  : " + wind31);
    System.out.println("Wind size: " +
		       JuiceUtil.arrayToString(windSize3, ", "));
    System.out.println("Anchor: "     +
		       JuiceUtil.arrayToString(anchor3  , ", "));

    for     (int i2 = 0; i2 < windSize3[2]; i2++)
      for   (int i1 = 0; i1 < windSize3[1]; i1++)
	for (int i0 = 0; i0 < windSize3[0]; i0++)
	  if(wind31.d[i2][i1][i0] !=       (i0+anchor3[0]) +
		                      10f *(i1+anchor3[1]) +
		        	      100f*(i2+anchor3[2])) {
            passed = false;
          }

    /* 
    System.out.println("test tentweight: \n");
    Rsf3  tent31 = (Rsf3)      myWind3.getWindowTent();
    float o40   = ((RsfSpace) tent31.getSpace()).getAxes()[0].offset; 
    float o41   = ((RsfSpace) tent31.getSpace()).getAxes()[1].offset; 
    float o42   = ((RsfSpace) tent31.getSpace()).getAxes()[2].offset; 
    if (tent31.d[0][0][0] !=  0.0625F) passed = false;
    if (tent31.d[1][1][1] !=  0.1250F) passed = false;
    // check that the offset changed!
    if (o40 != o30) passed = false; 
    if (o41 != o31) passed = false; 
    if (o42 != o32) passed = false; 
    System.out.println(tent31);
    System.out.println(tent31.d[1][1][1]);
    */

    System.out.println("test moving setting the anchor");

    // move the offset 
    int[] anchor4    = {0,0,0};
    myWind3.setAnchor(anchor4);
    Rsf3 wind32 = (Rsf3) myWind3.image(wall31);
    float o50   = ((RsfSpace) wind32.getSpace()).getAxes()[0].offset; 
    float o51   = ((RsfSpace) wind32.getSpace()).getAxes()[1].offset; 
    float o52   = ((RsfSpace) wind32.getSpace()).getAxes()[2].offset; 

    // compare values and offset (before and after)

    for     (int i2 = 0; i2 < windSize3[2]; i2++)
      for   (int i1 = 0; i1 < windSize3[1]; i1++)
	for (int i0 = 0; i0 < windSize3[0]; i0++)
	  if(wind32.d[i2][i1][i0] !=       (i0+anchor4[0]) +
		                      10f *(i1+anchor4[1]) +
		        	      100f*(i2+anchor4[2])) {
            passed = false;
          }
    // since I chose the first window the offsets should be the same again 
    if (o50 != o20) passed = false; 
    if (o51 != o21) passed = false; 
    if (o52 != o22) passed = false; 

    System.out.println("WindRsf : " + wind32);
    System.out.println("The two offsets should be the same:");
    System.out.println("wall3  offsets: " + o20 + " " + o21 + " " + o22); 
    System.out.println("wind32 offsets: " + o50 + " " + o51 + " " + o52);

    /*
    System.out.println("tent has new offsets");
    System.out.println("But the weights were not recomputed");

    Rsf3 tent32 = (Rsf3) myWind3.getWindowTent();
    float o60   = ((RsfSpace) tent32.getSpace()).getAxes()[0].offset; 
    float o61   = ((RsfSpace) tent32.getSpace()).getAxes()[1].offset; 
    float o62   = ((RsfSpace) tent32.getSpace()).getAxes()[2].offset; 
    if (o60 != o20) passed = false; 
    if (o61 != o21) passed = false; 
    if (o62 != o22) passed = false; 

    System.out.println("The two offsets should be the same:");
    System.out.println("wall3  offsets: " + o20 + " " + o21 + " " + o22); 
    System.out.println("wind32 offsets: " + o60 + " " + o61 + " " + o62);
    */ 

    // test the adjoint

    Rsf3 wall32 = (Rsf3) myWind3.adjoint().image(wind31);
    System.out.println(wall32); 
    if (wall32.d[0][0][0] != 21f) passed = false; 
    if (wall32.d[0][1][1] != 32f) passed = false; 
    if (wall32.d[4][3][2] != 00f) passed = false; 

    /*
    Rsf3 tent33 = (Rsf3) ((Window) myWind3.adjoint()).getWindowTent();
    float o70   = ((RsfSpace) tent33.getSpace()).getAxes()[0].offset; 
    float o71   = ((RsfSpace) tent33.getSpace()).getAxes()[1].offset; 
    float o72   = ((RsfSpace) tent33.getSpace()).getAxes()[2].offset; 
    if (o70 != o20) passed = false; 
    if (o71 != o21) passed = false; 
    if (o72 != o22) passed = false; 
    System.out.println(tent33);
    */
    Tester.exit(passed);
  }
}

