package  nstat.patchF;

import    sep.operator.SepLinearOperator;
import 	  jam.vector.*;
import 	  jam.operator.*;
import    rsf.vector.Axis;
import 	  rsfF.vector.*;

/**
 * PatchFactory creates a Patch operator.<BR>
 * This includes the forward (truncation) and adjoint (zero-padding).
 * <BR><!--#include file="./source.inc"--><BR>
 */
public class PatchFactory {
  protected Patch fwd;

  /**
   * @param quiltSpace the space of the Rsf from which a patch is taken
   * @param anchor     the index into the quiltSpace of the "smallest" vertex
   * @param patchSize  the dimensions of the patch
   */
  public PatchFactory(RsfSpace quiltSpace,
		      int[]    anchor    ,
		      int[]    patchSize ) {
    RsfSpace patchSpace = createPatchSpace(quiltSpace, anchor, patchSize);

    fwd = new Patch(quiltSpace, patchSpace, anchor, true );
  }

  RsfSpace createPatchSpace(RsfSpace quiltSpace,
			    int[]    anchor    ,
			    int[]    patchSize ) {
    Axis[]  quiltAxes = quiltSpace.getAxes();
    int n = quiltAxes.length;
    Axis[]  patchAxes = new Axis[n];
    for (int i = 0 ; i < n ; i++ ) {
      patchAxes[i] = new Axis(quiltAxes[i]);
      patchAxes[i].nsamples = patchSize[i];
      patchAxes[i].offset  += quiltAxes[i].delta * anchor[i];
    }
    return new RsfSpace(patchAxes);
  }

  /**
   * @returns the forward form of the patching operator, truncation.
   */
  public hasAdjoint getForward() { return fwd; }
  /**
   * @returns the adjoint form of the patching operator, zero-padding.
   */
  public hasAdjoint getAdjoint() { return fwd.adjoint(); }
}
