package nstat.tvdeconC;

import   jam.vector.*;
import   rsfF.vector.*;
import   sep.operator.SepLinearOperator;
import   jam.JamObject;

/**
 *  Nonstationary internal convolution, adjoint returns filter
 * <BR><!--#include file="./source.inc"--><BR>
 *
 * @author    Joel Schroeder
 */
public class Nicaf extends SepLinearOperator {
  protected Rsf x;

  public final static int filterSpace = 0;
  public final static int outputSpace = 1;

  // constructors
  /**
   * @param input Rsf of input data
   * @param spc   must equal filterSpace or outputSpace
   * @param space filter or output space of the convolution (according to spc)
   */
  public Nicaf(Rsf input, int spc, RsfSpace space) {
    super();
    switch (spc) {
    case filterSpace:
      setInput (space);
      setOutput(NonstationaryInternalConvolution
		.createOutputSpace((RsfSpace) input.getSpace(), space));
      break;
    case outputSpace:
      setInput (NonstationaryInternalConvolution
		.createFilterSpace((RsfSpace) input.getSpace(), space));
      setOutput(space);
      break;
    default:
      System.err.println("Error in Nicaf constructor:\n" +
			 "Integer argument, " + spc + ", is not one of: " +
			 "(filterSpace, outputSpace).");
      return;
    }
    this.x   = input;
  }

  /** copy constructor */
  protected Nicaf(Nicaf src) {
    super(src);
    this.x = (Rsf) src.x.copy();
  }

  public boolean equals(Object obj2) {
    if (this == obj2) return true;
    if (! (obj2 instanceof Nicaf)) return false;
    return (super.equals((Nicaf) obj2) &&
	    this.x.equals(((Nicaf) obj2).x));
  }

  public JamObject copy() { return new Nicaf(this); }

  public void apply( boolean add, Vector f, Vector y) {
    int result;
    if (isFwd)
      result = NonstationaryInternalConvolution.kOutput;
    else
      result = NonstationaryInternalConvolution.kFilter;
    NonstationaryInternalConvolution.staticApply(add, x , (Rsf) f , (Rsf) y ,
						 result);
  }

  // access to the input
  public Rsf  getInput  () { return x; }
  
  // I/O
  public String toString() {
    return ( "Nicaf (" + (isFwd ? "forward":"adjoint") + "):\n" +
	     "Domain:\n" + getDomain() + "\n" +
	     "Range :\n" + getRange () + "\n" +
	     "Input: \n" + getInput ()); }
}

