package nstat.tvdeconC;

import   jam.vector.*;
import   jam.operator.*;
import   rsf.vector.Axis;
import   rsfF.vector.*;
import   jam.JamObject;
import   sep.operator.SepBiLinearOperator;

/**
 * Nonstationary internal convolution (as a bilinear operator)<BR>
 * Uses C code for the calculation routine.
 * CURRENTLY WORKS ONLY FOR 1D<BR>
 * NICAI is not yet implemented (only NICAF).
 * <BR><!--#include file="./source.inc"--><BR>
 *
 * @see       InternalConvolution
 * @author    Joel Schroeder
 */
public class NonstationaryInternalConvolution extends SepBiLinearOperator {
  /**
   * These match up with kInput1, kInput2, and kOutput of
   * SepBiLinearOperator
   */
  public static final int kInput  = 0;
  public static final int kFilter = 1;
  public static final int kOutput = 2;

  static { // Load the native routines
    System.load("niclib.so");
  }

  public NonstationaryInternalConvolution(int spc1, RsfSpace space1,
					  int spc2, RsfSpace space2) {
    super(null, null, null);

    if ((spc1 != spc2)
	&& setSpace(spc1, space1)
	&& setSpace(spc2, space2)) {
      if      (getRange()   == null)
	setRange(   createOutputSpace((RsfSpace) getDomain1(),
				      (RsfSpace) getDomain2()));
      else if (getDomain1() == null)
	setDomain1( createInputSpace ((RsfSpace) getDomain2(),
				      (RsfSpace) getRange()));
      else
	setDomain2( createFilterSpace((RsfSpace) getDomain1(),
				      (RsfSpace) getRange()));
      return;
    }

    System.err.println("Error: integers to NonstationaryInternalConvolution" + 
		       " constructor must be two of:"           + "\n" +
		       " kInput, kFilter, kOutput." + "\n" ); 
  }

  /** copy constructor */
  protected
  NonstationaryInternalConvolution( NonstationaryInternalConvolution
				    src) {
    super(src);
  }

  /** The domain and range spaces are shallow copies */
  public JamObject copy() {
    return new NonstationaryInternalConvolution(this);
  }

  public boolean equals(Object obj2) {
    if (this == obj2) return true;
    if (!(obj2 instanceof NonstationaryInternalConvolution )) return false;
    return super.equals( (NonstationaryInternalConvolution) obj2);
  }

  public Operator getOp1 (Vector x1) { return getNicaf((Rsf) x1); }
  public Operator getOp2 (Vector x2) { return getNicai((Rsf) x2); }

  public Nicaf        getNicaf(Rsf input) {
    if (!(getDomain1().hasMember(input)))
      System.err.println("Error: Vector is not in domain1\n");
    return new Nicaf(input,  Nicaf.filterSpace, (RsfSpace) getDomain2());
  }

  public Nicai        getNicai(Rsf filter) {
    if (!(getDomain2().hasMember(filter)))
      System.err.println("Error: Vector is not in domain2\n");
    return new Nicai(filter, Nicai.inputSpace , (RsfSpace) getDomain1());
  }

  /**
   * <STRONG>WARNING: not yet implemented</STRONG>
   */
  public hasAdjoint getPartialAdjoint () {
    System.err.println( "Error: NonstationaryInternalConvolution" +
			".getPartialAdjoint() not yet implemented.\n" );
    return null; }

  // I/O
  public String toString() {
    return ( "Nonstationary Internal Convolution:"     + "\n" +
	     "Domain1:\n" + getDomain1() + "\n" +
	     "Domain2:\n" + getDomain2() + "\n" +
	     "Range  :\n" + getRange  () );
  }

  protected void apply(boolean add, Vector x, Vector f,
		       Vector y, int result) {
    staticApply(add, x, f, y, result);
  }

  static void staticApply(boolean add, Vector xv, Vector fv,
			  Vector yv, int result) {
    switch (result) {
    case kOutput:
      if (!(add)) yv.zero();
      break;
    case kInput :
      if (!(add)) xv.zero();
      break;
    case kFilter:
      if (!(add)) fv.zero();
      break;
    default:
      System.err.println( "Error: Integer argument to apply() not among " +
			  "options.");
      return;
    }

    int fn = ((RsfSpace) fv.getSpace()).getNSamples(1);
    int yn = ((RsfSpace) fv.getSpace()).getNSamples(0);

    applyC( add ? 1:0
	   , result
	   , ((Rsf) xv).getData()
	   , ((Rsf) fv).getData(), ((Rsf) yv).getData()
	   ,        fn          ,         yn);
  }

  static public native void applyC(int add, int result,
				   float[]  x,
				   float[]  f, float[]  y,
				   int     nf, int     ny);

  static RsfSpace createInputSpace (RsfSpace filtspc ,
				    RsfSpace oputspc ) {
    int  n    = oputspc.getDimension();
    if ( n*2 != filtspc.getDimension())
      System.err.println( "Error constructing filter space for " +
			  "non-stationary convolution:\n" +
			  "Number of filter axes is not twice that of input" );
    Axis[] inputAxes = new Axis[ n];
    for ( int i = 0 ; i < n ; i++ ) {
      inputAxes[i] = new Axis( ((RsfSpace) oputspc).getAxes()[i] );
      inputAxes[i].nsamples = ( filtspc.getNSamples(i  ) +
				filtspc.getNSamples(i+n) - 1); }
    return new RsfSpace( inputAxes ); }

  static RsfSpace createFilterSpace(RsfSpace iputspc ,
				    RsfSpace oputspc ) {
    int n  = oputspc.getDimension();
    if (n != oputspc.getDimension())
      System.err.println( "Error constructing filter space for " +
			  "non-stationary convolution:\n" +
			  "Number of input and output axes not equal." );
    int[] filterNSamples = new int[ n*2];
    for ( int i = 0 ; i < n ; i++ ) {
      filterNSamples[i]   =  oputspc.getNSamples(i);
      filterNSamples[i+n] = (iputspc.getNSamples(i) -
			     oputspc.getNSamples(i) + 1); }
    return new RsfSpace( filterNSamples ); }

  static RsfSpace createOutputSpace(RsfSpace iputspc ,
				    RsfSpace filtspc ) {
    int  n    = iputspc.getDimension();
    if ( n*2 != filtspc.getDimension())
      System.err.println( "Error constructing filter space for " +
			  "non-stationary convolution:\n" +
			  "Number of filter axes is not twice that of input" );
    Axis[] outputAxes = new Axis[ n];
    for ( int i = 0 ; i < n ; i++ ) {
      outputAxes[i] = new Axis( ((RsfSpace) iputspc).getAxes()[i] );
      outputAxes[i].nsamples = filtspc.getNSamples(i); }
    return new RsfSpace( outputAxes); }
}

