package nstat.tvdeconC;

import  nstat.tvdeconF.RsfHistoryIterator;
import  nstat.tvdeconF.LeakyIntegration;
import  nstat.tvdeconF.PefFreeMask;

import 	 jam.vector.*;
import 	 jam.operator.*;
import 	 jam.solver.*;
import juice.vector.*;
import juice.operator.*;
import juice.solver.*;
import 	rsfF.vector.*;
import   sep.io.*;

/**
 * <BR><!--#include file="./source.inc"--><BR>
 */
public class TVDecon {
  public static void main(String[] args) {
    ArgumentKeeper argKeeper = new ArgumentKeeper(args);

    int	  filtSize = argKeeper.getInt  ("filtSize", "8"   );
    int   filtGap  = argKeeper.getInt  ("filtGap ", "5"   );
    float rho      = argKeeper.getFloat("rho"     , "0.9F");
    float eps      = argKeeper.getFloat("eps"     , "0.0F");

    float tol      = argKeeper.getFloat("tol"     , "1e-4");
    int   niter    = argKeeper.getInt  ("niter"   , "15"  );

    Rsf in       = new Rsf(argKeeper.getString("in"));
    int inptSize =   ((RsfSpace)   in.getSpace()).getNSamples(0);

    int otptSize = inptSize - filtSize + 1;
    Nicaf
      nicaf = new Nicaf(in, Nicaf.outputSpace, new RsfSpace(otptSize));

    int[] spot = { otptSize - 1, filtGap };
    PefFreeMask
      mask  = new PefFreeMask     ((RsfSpace) nicaf.getDomain(), spot);

    LeakyIntegration
      leaky = new LeakyIntegration((RsfSpace) nicaf.getDomain(), rho);

    CompoundLinearOperatorAdjoint
      YKS   = new CompoundLinOpAdj(leaky, mask, nicaf);

    ScaledLinearOperatorAdjoint
      regularizer = new ScaledLinOpAdj(
		        new IdentityOp(nicaf.getDomain()),
			eps);

    // Create and initialize m = 1      0    0 	 ...
    //                           1-rho  0    0 	 ...
    //                           1-rho  0    0 	 ...
    //                           ...   ...  ...  ...
    Rsf m = new Rsf((RsfSpace) YKS.getDomain());
    m.zero();
    int[] mi = { 0, 0 };
    m.setDatum(mi, 1F);
    for (mi[0] = 1; mi[0] < ((RsfSpace) m.getSpace()).getNSamples(0); mi[0]++)
      m.setDatum(mi, 1F - rho);

    Vector r0 = nicaf.image(leaky.image(m));
    Vector r1 = regularizer.getRange().newMember();
    r1.zero();
    Vector negR0 = r0.getSpace().newMember();
    negR0.neg(r0);
    ProductVector r = new ProductVec(negR0, r1);

    BlockLinearOperatorAdjoint colOp = new BlockLinOpAdj(2, 1);
    colOp.setOp(0, 0, YKS);
    colOp.setOp(1, 0, regularizer);

    Terminator 	 term	      = new TerminatorSimple(niter, tol);
    Iterator   	 iter	      = new RsfHistoryIterator(YKS, (Rsf) r0, 5, term);
    CGSolver   	 CGsolve      = new CGSolver(iter);    

    NormalEqns nrmeq = new NormalEqns(colOp, r);
    CGsolve.solve(nrmeq.getAtA(), nrmeq.getAtb(), m);

    Rsf out = ((RsfHistoryIterator) iter).getHistory();
    
    out.write(argKeeper.getString("out"));
  }
}
