package nstat.tvdeconF;

import   jam.vector.*;
import   rsfF.vector.*;
import   sep.operator.SepLinearOperator;
import   jam.JamObject;

/**
 * Leaky Integration over a specified dimension.
 * <BR><!--#include file="./source.inc"--><BR>
 *
 * @author    Joel Schroeder
 */
public class LeakyIntegration extends SepLinearOperator {
  protected int      dim;
  protected float    rho;

  /**
   * @param space     is the domain and range of the leaky integration
   * @param dimension is the dimension over which to perform the integration
   * @param rho       sets the rate of decay, and should be between 0 and 1 (1 = causal integration)
   */
  public LeakyIntegration(RsfSpace space, int dimension, float rho) {
    super(space);
    this.dim = dimension;
    this.rho = rho;
  }

  /** copy constructor */
  protected LeakyIntegration(LeakyIntegration src) {
    super(src);
    this.dim = src.dim;
    this.rho = src.rho;
  }

  /**
   * Perform the integration over the first dimension
   */
  public LeakyIntegration( RsfSpace space, float rho) {
    this(space, 0, rho);
  }

  public boolean equals(Object obj2) {
    if (this == obj2) return true;
    if (! (obj2 instanceof LeakyIntegration)) return false;
    return (this.dim == ((LeakyIntegration) obj2).dim &&
	    this.rho == ((LeakyIntegration) obj2).rho &&
	    super.equals((LeakyIntegration) obj2));
  }

  public JamObject copy() { return new LeakyIntegration(this); }

  public void apply(boolean add, Vector x, Vector y) {
    if (!add) { if (isFwd) y.zero(); else x.zero(); }
    float[] xx = ((Rsf) x).getData();
    float[] yy = ((Rsf) y).getData();
    int nPre = ((RsfSpace) this.input).getNSamples(0, dim - 1);
    int n    = ((RsfSpace) this.input).getNSamples(dim);
    int nPst = ((RsfSpace) this.input).getNSamples(dim + 1,
						   ((RsfSpace) input)
						   .getDimension() - 1);
    int i = 0;
    for (int pst = 0; pst < nPst; pst++) {
    for (int pre = 0; pre < nPre; pre++) {
      i = pst * n * nPre + pre;
      float temp = 0F;
      if (isFwd) {
	for ( int j = 0  ; j < n ; j++) {
	  temp   = temp * rho + xx[i];
	  yy[i] += temp;
	  i += nPre; }}
      else {
	i += nPre * (n-1);
	for ( int j = n-1; j >= 0; j--) {
	  temp   = temp * rho + yy[i];
	  xx[i] += temp;
	  i -= nPre; }}
    }}
  }

  public float getRho()          { return rho; }
  public void  setRho(float rho) {
    this.rho = rho;
    if (adj != null)
      ((LeakyIntegration) adj).rho = rho;
  }
  public int   getDim()          { return dim; }
  public void  setDim(int   dim) {
    this.dim = dim;
    if (adj != null)
      ((LeakyIntegration) adj).dim = dim;
  }
  
  // I/O
  public String toString() {
    return ( "LeakyIntegration (" + (isFwd ? "forward":"adjoint") + "):\n" +
	     "Rho = " + rho + "\nDimension = " + dim + "\n" +
	     "Space:\n" + getDomain()); }
}

