package nstat.tvdeconF;

import   jam.vector.*;
import   sep.operator.SepLinearOperator;
import   rsfF.vector.*;

/**
 * Patch inserts/removes an Rsf into/from an Rsf of one more dimension.<BR>
 * <BR><!--#include file="./source.inc"--><BR>
 * For example, it inserts a 2D Rsf into a 3D Rsf.  The 3D Rsf can be thought
 * of as a sequence of 2D Rsfs.
 *
 * @author    Joel Schroeder
 */
public class Patch extends SepLinearOperator {
  protected int index;

  /**
   * @param input         The input RsfSpace
   * @param sequenceSize  Specifies the output space as <tt>input X RsfSpace(sequenceSize)</tt>
   * @param sequenceIndex Specifies which slot in the output to insert the input
   */
  public Patch( RsfSpace input, int sequenceSize, int sequenceIndex) {
    super(input, input.cross(new RsfSpace(sequenceSize)));
    index  = sequenceIndex;
  }

  /**
   * @param input         The input RsfSpace
   * @param sequenceSize  Specifies the output space as <tt>input X RsfSpace(sequenceSize)</tt>
   * The patch index gets tha value of zero
   */
  public Patch(RsfSpace input, int sequenceSize) {
    this(input, sequenceSize, 0);
  }

  public boolean equals( Object patch2) {
    return ((patch2 instanceof Patch)
	    && super.equals(         patch2)
	    && index == ((Patch) patch2).index );
  }

  public void apply( boolean add, Vector i, Vector o) {
    Rsf dom = isFwd ? ((Rsf) i):((Rsf) o);
    Rsf ran = isFwd ? ((Rsf) o):((Rsf) i);
    if (!add) ran.zero();
    float[] domData = dom.getData();
    float[] ranData = ran.getData();
    int patchSize = ((Rsf) i).getTotalNSamples();
    int domInd = isFwd ? 0:(patchSize * index);
    int ranInd = isFwd ? (patchSize * index):0;
    for ( int ind = 0; ind < patchSize; ind++, domInd++, ranInd++ )
      ranData[ranInd] += domData[domInd];
  }

  // access to the index of the patch
  public int  getIndex  () { return index; }
  public void setIndex  (int index) {
    this.index = index;
    if (adj != null)
      ((Patch) adj).index = index;
  }

  public jam.JamObject copy() {
    System.err.println("Error: Patch.copy() not implemented.\n");
    return null;
  }
  
  // I/O
  public String toString() {
    return ( "Patch (" + (isFwd ? "forward":"adjoint") + "):\n" +
	     "Domain:\n" + getDomain() + "\n" +
	     "Range :\n" + getRange () + "\n" +
	     "Patch Index: " + index ); }
}


