package nstat.tvdeconF;

import   jam.operator.*;
import   jam.solver.*;
import   jam.vector.*;
import juice.solver.*;
import   rsfF.vector.*;

/**
 * <BR><!--#include file="./source.inc"--><BR>
 * 
 * @author      Joel Schroeder
 */
public class RsfHistoryIterator implements Iterator { 
  IterativeSolver solver;
  Vector	  history;
  Vector          historyStep;
  int 		  memorySize;
  Operator        historyOp;
  Rsf             b;
  Patch           patch;

  Terminator      terminator;

  public RsfHistoryIterator(Operator historyOp, Rsf b, int memorySize,
			    Terminator terminator) {
    this.historyOp   = historyOp;
    this.b           = b;
    this.memorySize  = memorySize;
    this.patch       = new Patch((RsfSpace) historyOp.getRange(),
				 memorySize + 1);
    this.history     = patch.getRange ().newMember();
    this.historyStep = patch.getDomain().newMember();
    history.zero();
    this.terminator  = terminator;
  }

  public void    setSolver(IterativeSolver solver) {
    this.solver = solver;
    terminator.setSolver(solver);
  }

  void appendHistory(int slotNum) {
    patch.setIndex(slotNum);
    historyStep.add(historyOp.image(solver.getApproximation()), b);
    patch.addImage(historyStep, history);
  }

  public void    start() { appendHistory(solver.getIteration()); }

  public boolean hasMoreIterations() {
    return terminator.hasMoreIterations();
  }

  public void    nextIteration() {
    int i = solver.getIteration() + 1;
    if (i < memorySize)
      appendHistory(i);
    System.out.println("iter = " + i + "\n" +
		       "\tNorm(Residual) = " +
		       Math.sqrt(solver.getNormResidual()));
  }

  public void    finish()     { appendHistory(memorySize); }

  public Rsf     getHistory() { return (Rsf) history; }
}
